import os
import numpy as np
import torch
from torch.utils.data import Dataset, DataLoader, ConcatDataset

class BatDataset2d(Dataset):
    def __init__(self, csv_file, stride, H, W, Wy):
        for Wc in W:
            assert len(Wc) == len(W[0]), "BatDataset2d: W has Wrong Dimensions"
        self.file = np.loadtxt(csv_file, delimiter=",")
        self.stride = stride
        self.H = H
        self.W = W
        self.Wy = Wy
    def __getitem__(self, idx):
        X = np.empty((len(self.W), self.H, len(self.W[0])), float)
        for i, Wc in enumerate(self.W):
            X[i, :, :] = self.file[idx*self.stride:idx*self.stride+self.H, Wc]
        X = torch.from_numpy(X).float()
        Y = torch.from_numpy(self.file[idx*self.stride, self.Wy]).float()
        return X, Y
    def __len__(self):
        return (len(self.file)-self.H+1)//self.stride

class BatDataLoader2d(DataLoader):
    def __init__(self, list_file, shuffle, stride, N, H, *W):
        assert len(W) > 1, "BatDataset2d: W should have at least 2 lists for X and Y"
        self.list_file = list_file
        self.shuffle = shuffle
        self.stride = stride
        self.N = N
        self.H = H
        self.W = W[:-1]
        self.Wy = W[-1]
        self.dataset = []
        path = os.path.split(list_file)[0]
        if len(path) == 0:
            path = "."
        lines = open(list_file, "r").readlines()
        for line in lines:
            line = path + "/" + line.strip()
            if os.path.isfile(line):
                self.dataset.append(BatDataset2d(line, self.stride, self.H, self.W, self.Wy))
        super(BatDataLoader2d, self).__init__(
                dataset=ConcatDataset(self.dataset), batch_size=self.N, shuffle = self.shuffle)

class BatDataset1d(Dataset):
    def __init__(self, csv_file, stride, H, W, Wy):
        #assert len(W[0]) == 1, "BatDataset1d: W has Wrong Dimensions"
        self.file = np.loadtxt(csv_file, delimiter=",")
        self.stride = stride
        self.H = H
        self.W = W
        self.L = H*len(self.W[0])
        self.Wy = Wy
    def __getitem__(self, idx):
        X = np.empty((len(self.W), self.L), float)
        for i, Wc in enumerate(self.W):
            X[i, :] = np.reshape(self.file[idx*self.stride:idx*self.stride+self.H, Wc], self.L)
        X = torch.from_numpy(X).float()
        Y = torch.from_numpy(self.file[idx*self.stride, self.Wy]).float()
        return X, Y
    def __len__(self):
        return (len(self.file)-self.H+1)//self.stride

class BatDataLoader1d(DataLoader):
    def __init__(self, list_file, shuffle, stride, N, H, *W):
        assert len(W) > 1, "BatDataset1d: W should have at least 2 lists for X and Y"
        self.list_file = list_file
        self.shuffle = shuffle
        self.stride = stride
        self.N = N
        self.H = H
        self.W = W[:-1]
        self.Wy = W[-1]
        self.dataset = []
        path = os.path.split(list_file)[0]
        if len(path) == 0:
            path = "."
        lines = open(list_file, "r").readlines()
        for line in lines:
            line = path + "/" + line.strip()
            if os.path.isfile(line):
                self.dataset.append(BatDataset1d(line, self.stride, self.H, self.W, self.Wy))
        super(BatDataLoader1d, self).__init__(
                dataset=ConcatDataset(self.dataset), batch_size=self.N, shuffle = self.shuffle)

if __name__ == "__main__":
    stride = 2
    N = 3
    H = 5
    #W1 = [0, 1]
    #W2 = [1, 2]
    #W3 = [0, 2]
    #loader = BatDataLoader2d("tst_list.txt", False, stride, N, H, W1, W2, W3)
    W1 = [0]
    W2 = [1]
    W3 = [2]
    loader = BatDataLoader1d("tst_list.txt", False, stride, N, H, W1, W2, W3)
    it = iter(loader)
    batch = it.next()
    print(batch)

